from .match_format import MatchFormat
from .services.cricinfo_service import CricinfoService
from .helpers.stat_type import StatType
from .team import Team
import pandas as pd

class Cricinfo:
    """
    Loads cricket statistics from ESPN's cricinfo site into Pandas data frames.
    """ 
    @staticmethod
    def retrieve_batting_stats(team: Team, match_format: MatchFormat) -> pd.DataFrame :
        """
        Retrieve the batting statistics for all players in international cricket.

        :param team: Filter stats by team. 
        :type team: Team, :obj:`None`

        :param match_format: Filter stats by format.
        :type match_format: MatchFormat

        :raises TypeError: If Team or MatchFormat do not use the specified type.
        """
        return CricinfoService.retrieve_stats(team, match_format, StatType.BATTING)
    
    @staticmethod
    def retrieve_bowling_stats(team: Team, match_format: MatchFormat) -> pd.DataFrame:
        """
        Retrieve the bowling statistics for all players in international cricket.

        :param team: Filter stats by team. 
        :type team: Team, :obj:`None`

        :param match_format: Filter stats by format.
        :type match_format: MatchFormat

        :raises TypeError: If Team or MatchFormat do not use the specified type.
        """
        return CricinfoService.retrieve_stats(team, match_format, StatType.BOWLING)
    
    @staticmethod
    def retrieve_fielding_stats(team: Team, match_format: MatchFormat) -> pd.DataFrame:
        """
        Retrieve the fielding statistics for all players in international cricket.

        :param team: Filter stats by team. 
        :type team: Team, :obj:`None`

        :param match_format: Filter stats by format.
        :type match_format: MatchFormat

        :raises TypeError: If Team or MatchFormat do not use the specified type.
        """
        return CricinfoService.retrieve_stats(team, match_format, StatType.FIELDING)

    @staticmethod
    def retrieve_allround_stats(team: Team, match_format: MatchFormat) -> pd.DataFrame:
        """
        Retrieve the all-round statistics for all players in international cricket.

        :param team: Filter stats by team. 
        :type team: Team, :obj:`None`

        :param match_format: Filter stats by format.
        :type match_format: MatchFormat

        :raises TypeError: If Team or MatchFormat do not use the specified type.
        """
        return CricinfoService.retrieve_stats(team, match_format, StatType.ALLROUND)

    @staticmethod
    def retrieve_partnership_stats(team: Team, match_format: MatchFormat) -> pd.DataFrame:
        """
        Retrieve the partnership statistics for all partnerships for a given team in international cricket.

        :param team: Filter stats by team. 
        :type team: Team, :obj:`None`

        :param match_format: Filter stats by format.
        :type match_format: MatchFormat

        :raises TypeError: If Team or MatchFormat do not use the specified type.
        """
        return CricinfoService.retrieve_stats(team, match_format, StatType.PARTNERSHIP)

    @staticmethod
    def retrieve_team_stats(team: Team, match_format: MatchFormat) -> pd.DataFrame:
        """
        Retrieve team statistics for a given team in international cricket.

        :param team: Filter stats by team. 
        :type team: Team, :obj:`None`

        :param match_format: Filter stats by format.
        :type match_format: MatchFormat

        :raises TypeError: If Team or MatchFormat do not use the specified type.
        """
        return CricinfoService.retrieve_stats(team, match_format, StatType.TEAM)

    @staticmethod
    def retrieve_aggregate_stats(team: Team, match_format: MatchFormat) -> pd.DataFrame:
        """
        Retrieve aggregated team statistics for a given team in international cricket.

        :param team: Filter stats by team. 
        :type team: Team, :obj:`None`

        :param match_format: Filter stats by format.
        :type match_format: MatchFormat

        :raises TypeError: If Team or MatchFormat do not use the specified type.
        """
        return CricinfoService.retrieve_stats(team, match_format, StatType.AGGREGATE)   


from cricinfo import MatchFormat 

class TestMatchFormats:

    def test_match_formats(self):
        match_formats = [
            "Test",
            "ODI",
            "T20I",
            "International"
        ]
        for match_format in match_formats:
            assert match_format in MatchFormat.__members__


from enum import Enum

class RequestHelper(Enum):
    """
    Constants used when making HTTP requests to cricinfo
    """

    PAGE_PARAMETER = "page"
    TEAM_PARAMETER = "team"
    TEMPLATE_PARAMETER = "template"
    TEMPLATE_VALUE = "results"
    TYPE_PARAMETER = "type"
    MATCH_FORMAT_PARAMETER = "class"
    USER_AGENT_PARAMEATER = "User-Agent"
    USER_AGENT_VALUE = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36"
    REQUEST_URL='https://stats.espncricinfo.com/ci/engine/stats/index.html'

    @classmethod
    def HEADER_MAP(cls):
        """
        Populates the User Agent Header
        """
        return {cls.USER_AGENT_PARAMEATER.value : cls.USER_AGENT_VALUE.value}



from enum import Enum

class MatchFormat(Enum):
    """
    Statistics can be filtered by the format of the match.
    International aggregates the stats from the 3 formats.
    """
    Test = '1'
    ODI = '2'
    T20I = '3'
    International = '11'


import pandas as pd
import requests
from ..helpers.data_sanitizer import DataSanatizer
from ..helpers.request_helper import RequestHelper
from ..helpers.stat_type import StatType
from ..match_format import MatchFormat
from ..team import Team

class CricinfoService:
    @staticmethod
    def retrieve_stats(team: Team, match_format: MatchFormat, stats_type: StatType) -> pd.DataFrame:
        CricinfoService._validate_request(team, match_format, stats_type)
        params = CricinfoService._construct_query_parameters(team=team, match_format=match_format, stats_type=stats_type)
        dataframes = []
        tables = CricinfoService._parse_page(params=params, page=1, dataframes=dataframes)
        num_pages = int(tables[1][0][0].split(' ')[-1])
        for page in range(2, num_pages+1):
            CricinfoService._parse_page(params=params, page=page, dataframes=dataframes)
        return pd.concat(dataframes, ignore_index=True)

    @staticmethod
    def _construct_query_parameters(team: Team, match_format: MatchFormat, stats_type: StatType) -> dict:
        params = {
            RequestHelper.MATCH_FORMAT_PARAMETER.value : match_format.value,
            RequestHelper.TYPE_PARAMETER.value  : stats_type.value,
            RequestHelper.TEMPLATE_PARAMETER.value : RequestHelper.TEMPLATE_VALUE.value 
        }
        if team is not None:
            params[RequestHelper.TEAM_PARAMETER.value] = team.value
        return params
    
    @staticmethod
    def _parse_page(params: dict, page: int, dataframes: list[pd.DataFrame]):
        params[RequestHelper.PAGE_PARAMETER.value] = str(page)
        response = requests.get(RequestHelper.REQUEST_URL.value, headers=RequestHelper.HEADER_MAP(), params=params)
        tables = pd.read_html(response.content)
        dataframes.append(DataSanatizer._clean_nan_column(tables[2]))
        return tables
    
    @staticmethod
    def _validate_request(team, match_format, stats_type):
        if team is not None and not isinstance(team, Team):
            raise TypeError(f"Invalid type for team. Expected {Team.__name__}, got {type(team).__name__} instead.")
        
        if not isinstance(match_format, MatchFormat):
            raise TypeError(f"Invalid type for match_format. Expected {MatchFormat.__name__}, got {type(match_format).__name__} instead.")
        
        if not isinstance(stats_type, StatType):
            raise TypeError(f"Invalid type for stats_type. Expected {StatType.__name__}, got {type(stats_type).__name__} instead.")